#!/usr/bin/env python
# ex:ts=2

# (c) 2004 Tilman Linneweh
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the above copyright notice is
# retained.
#
# Getting started
# ###############
#
# * Type rssfetch.py -h to read the help message
# * Define a class rssitems in your stylesheet
#
# $Id: rssfetch.py,v 1.7 2005/04/09 11:32:08 arved Exp $

import feedparser
import re
from getopt import *
import sys

version = "0.2.1"

class HTMLEntityEncode:
	"encode a String with special characters to HTML"
	## htmlentityEncode
	# from comp.lang.python by Steffen Ries <steffen.ries@sympatico.ca>
	# in <m3k7yr9wci.fsf@gondolin.beleriand>

	string = ''

	def __init__(self, s=None):
		self.string = self.htmlentityEncode(s)

	_u2html = {}   # unicode to html mapping

	def _make_u2html(self):
    		from htmlentitydefs import entitydefs

    		def c2u(c):
        		if len(c) == 1:
            			return unicode(c, 'latin1')
        		if c.startswith('&#'):
            			return unichr(int(c[2:-1]))

    		for entity,val in entitydefs.items():
        		self._u2html[c2u(val)] = "&%s;" % entity

	def htmlentityEncode(self, s):
    		"""
    		convert unicode string s to ascii, replace non-ascii characters with
    		html entitydef or "?"
    		"""

    		if not self._u2html:
        		self._make_u2html()

    		l = [self._u2html.get(c, c) for c in s]

    		return ''.join(l).encode('ascii', 'replace')

## processFeed

class RSSFetch:

	def fixuptitle(self,s):
		if s == "Wibble": return "lauri - " + s
		if s == "Autoblogiographie": return "Hendrik Scholz - " + s
		if s == "Vortices of Extelligence": return "roam - " + s
		if s == "Cosmic Seriosity Balance": return "nbm - " + s
		s = re.sub("\(mailto:.*","",s)
		return s

	def HTMLFeedDetect(self,url):
		"Detect a Feed with HTML entities"
		regex = re.compile('.*(twoday|antville|fsinf|blogger.de).*')
		rxp = regex.search(url)
		if not rxp == None:
			return 1

	def processFeed(self,url,numberofitems,fw,adddate,debug):
		try:
			d = feedparser.parse(url)
		except:
			print "Could not parse feed: "+ url
			return -1
		htmlfeed = self.HTMLFeedDetect(url)
		if debug:
			print d.feed
		if d.feed.has_key('title'):
			if d.feed.has_key('author') and d.feed.author != "":
				htmltitle = d.feed.author + " - " + d.feed.title
			else:
				htmltitle = d.feed.title
			htmltitle = self.fixuptitle(htmltitle)
			if htmlfeed == None:
				htmltitle = HTMLEntityEncode(htmltitle).string
		else:
			if debug: print "Title not found"
			return -1
		if debug:
			print "<a href=\"" + d.feed.link + "\">" + htmltitle + "</a>"
		print >> fw, "<a href=\"" + d.feed.link + "\">" + htmltitle + "</a>"
		feeditems = len(d['entries'])
		if feeditems <= numberofitems:
			numberofitems = feeditems
		if debug: print "Number of items: "+str(numberofitems)
		print >> fw, "<div class=\"rssitems\">"
		for a in range(0,numberofitems):
			e = d.entries[a]
			if e.has_key('title'):
				if htmlfeed == None:
					htmlentry = HTMLEntityEncode(e.title).string
				else:
					htmlentry = e.title
			else:
				if debug: print "Entry name not found!"
				adddate = 1
				htmlentry = ""
			if adddate:
				if e.has_key('issued'):
					htmlentry = e.issued + " " + htmlentry
				elif e.has_key('created'):
					htmlentry = e.created + " " + htmlentry
				elif e.has_key('modified'):
					htmlentry = e.modified + " " + htmlentry
				else:
					if debug: print e
					continue
			if debug:
				print "<a href=\"" + e.link + "\">" + htmlentry + "</a>"
			print >> fw, "<a href=\"" + e.link + "\">" + htmlentry + "</a><br />"
		print >> fw, "</div>"
		return 0

## main

def main():
	adddate= 0
	numberofitems = 3
	infile = "blogroll.en"
	outfile = "blogroll.en.inc"
	uacomment = "http://arved.narf.at/weblog/"
	debug= 0

	opts,args=getopt(sys.argv[1:],'c:df:gho:u:vF:V')

	for o,a in opts:
		if o=="-c": numberofitems= int(a)
		if o=="-d": adddate=1
		if o=="-f": infile=a
		if o=="-g": debug=1
		if o=="-h":
			print "usage: $prog [-cdefhvV]"
			print "\t -c Number	Number of topics displayed"
			print "\t -d		Add date to topic"
			print "\t -F URL		feed"
			print "\t -f filename	configurationfile"
	 		print "\t -u comment	Useragent comment"
			print "\t -o filename	outputfile"
			print "\t -g		debugoutput"
			print "\t -h		summary of command line options"
			print "\t -v		verbose output"
			print "\t -V		print the version and exit"
			return
		if o=="-o": outfile=a
		if o=="-u": uacomment=a
		if o=="-v": debug=1
		if o=="-F":
			fw=open(outfile,"w")
			rf = RSSFetch()
			rf.processFeed(a,numberofitems,fw,adddate,debug)
			return
		if o=="-V":
			print "py-rssfetch " + version
			print "(c) 2004 Tilman Linneweh"
			return

	if debug:
		print "Running with debugging output"
	feedparser.USER_AGENT = "py-rssfetch/" + version + " - " + uacomment
	fr=open(infile,"r")
	fw=open(outfile,"w")
	for a in fr.readlines():
		if a[0] != "#":
			regex=re.compile("((((https?|ftp)://)|(mailto|file|news):)[^ \"\'<>;:,]+)",re.I)
			rxp = regex.search(a)
			if not rxp == None:
				a = rxp.group()
			if debug:
				print "Processing: " + a
			rf = RSSFetch()
			rf.processFeed(a,numberofitems,fw,adddate,debug)
	fr.close()
	fw.close()

if __name__ == "__main__":
    main()

